unit ACQUIRE; {Unit 13 of CONCRETE.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, ROOTS, Graph, Dos, Crt;

  procedure InverseRootsToCoeffs (p: integer;
                  var cartRoot, alpha: complexVector);

  procedure RootsAcquisition (var p, r: integer;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

  procedure PolarAcquisition (var p, r: integer;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

  procedure CoeffAcquisition (var g: integer;
                  var gamma: complexVector;
                  var coeffStatus: integer;
                  stability: boolean);

  procedure ParameterAcquisition (var p, r: integer;
                  var alpha: shortVector;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

 procedure MAverageAcquisition (var p, r: integer;
                  var alpha: shortVector;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

  procedure ARMAacquisition (var ARMASpec: ARMARecord;
                  stability: boolean);

  procedure LSDEAcquisition (var LSDESpec: ARMARecord;
                  stability: boolean);

  procedure GetLags (var lags: integer);

implementation

 {_________________________________________________}

  procedure InverseRootsToCoeffs (p: integer;
                  var cartRoot, alpha: complexVector);

    var
      j, k: integer;
      store: complex;

  begin {RootToCoefficient}
    alpha[0].re := 1.0;
    alpha[0].im := 0.0;

    for k := 1 to p do
      begin {k}
        alpha[k].im := 0.0;
        alpha[k].re := 0.0;
        for j := k downto 1 do
          begin {j}
            ComplexMult(cartRoot[k], alpha[j - 1], store);
            ComplexSub(alpha[j], store, alpha[j]);
          end; {j}
      end; {k}

  end; {InverseRootsToCoeffs}

   {_________________________________________________}

  procedure RootsAcquisition (var p, r: integer;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

    const
      virtualZero = 1E-12;

    var
      i: integer;
      pi, radToDeg, modulus: real;

  begin {Roots Acquisition}
    pi := 4 * ArcTan(1.0);
    radToDeg := 180 / pi;

    if (discontent = continuous) then
      stability := false
    else
      stability := true;

    Writeln;

    r := 0;
    if p > 1 then
      repeat
        Writeln('    How many pairs of complex conjugates?');
        GetBoundedInteger(0, 20, r);
      until (2 * r <= p) and (r >= 0);

    i := 1;
    while (i <= p) do
      begin {while}
        cartRoot[i].im := 0.0;
        if (i = 1) and (r > 0) then
          Writeln('    Complex conjugate roots')
        else if (i = 2 * r + 1) then
          Writeln('    Real roots');
        repeat {until Modulus < unity}
          Writeln('    Specify  root[', i : 1, '].re');
          if discontent = continuous then
            ReadBoundedNumber(-100, 0.0, cartRoot[i].re)
          else
            ReadANumber(cartRoot[i].re);
          if i <= 2 * r then
            begin
              Writeln('    Specify  root[', i : 1, '].im');
              ReadANumber(cartRoot[i].im);
              cartRoot[i + 1].re := cartRoot[i].re;
              cartRoot[i + 1].im := -cartRoot[i].im;
            end;
          modulus := cartRoot[i].re * cartRoot[i].re + cartRoot[i].im * cartRoot[i].im;
          if (stability and (modulus >= 1)) then
            begin
              Sysbeep(3);
              Writeln('    The modulus of the root must be less than unity');
            end;
        until (modulus < 1) or (stability = false);
        if i <= 2 * r then
          i := i + 2
        else
          i := i + 1;
      end; {while}

    Writeln;
    for i := 1 to p do
      begin
        Write('    root(', i : 1, ').re = ', cartRoot[i].re : 8 : 4);
        Writeln('    root(', i : 1, ').im = ', cartRoot[i].im : 8 : 4);
      end;
    Writeln;

    Writeln('    Press <RETURN> to continue');
    Readln;

    for i := 1 to p do
      begin {i}
        polarRoot[i].modu := Sqrt(Sqr(cartRoot[i].re) + Sqr(cartRoot[i].im));
        if Abs(cartRoot[i].re) < virtualZero then
          begin
            if cartRoot[i].im < 0 then
              polarRoot[i].argu := pi / 2
            else
              polarRoot[i].argu := -pi / 2
          end
        else if cartRoot[i].re < 0 then
          polarRoot[i].argu := ArcTan(cartRoot[i].im / cartRoot[i].re) + pi
        else
          polarRoot[i].argu := ArcTan(cartRoot[i].im / cartRoot[i].re);
        polarRoot[i].argu := polarRoot[i].argu * radToDeg;
      end; {i}

    Writeln;
    for i := 1 to p do
      begin
        Write('    root(', i : 1, ').modu = ', polarRoot[i].modu : 8 : 4);
        Writeln('    root(', i : 1, ').argu = ', polarRoot[i].argu : 8 : 4);
        Writeln;
      end;
    Writeln;

    Writeln('    Press <RETURN> to continue');
    Readln;

  end; {RootsAcquisition}

{_________________________________________________}

  procedure PolarAcquisition (var p, r: integer;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

    const
      virtualZero = 1E-12;

    var
      i: integer;
      degToRad, rho, theta: real;

  begin {Roots Acquisition}
    degToRad := ArcTan(1.0) / 45;

    r := 0;
    if p > 1 then
      repeat
        Writeln('    How many pairs of complex conjugates?');
        GetBoundedInteger(0, 20, r);
      until (2 * r <= p) and (r >= 0);

    i := 1;
    while (i <= p) do
      begin {while}
        if (i = 1) and (r > 0) then
          Writeln('    Complex conjugate roots')
        else if (i = 2 * r + 1) then
          Writeln('    Real roots');
        repeat
          if i <= 2 * r then
            begin
              Writeln('    Specify the argument omega[', i : 1, '] in degrees');
              ReadBoundedNumber(0.0, 180.0, polarRoot[i].argu);
              Writeln('    Specify the modulus rho[', i : 1, ']');
              ReadANumber(polarRoot[i].modu);
              polarRoot[i + 1].argu := -polarRoot[i].argu;
              polarRoot[i + 1].modu := polarRoot[i].modu;
            end
          else {if  i > 2 * r }
            begin
              Writeln('    Specify the real root: root[', i : 1, ']');
              ReadANumber(polarRoot[i].modu);
              polarRoot[i].argu := 0.0;
              if (polarRoot[i].modu < 0) then
                begin
                  polarRoot[i].modu := -polarRoot[i].modu;
                  polarRoot[i].argu := 180.0;
                end;
            end;
          if (stability and (sqr(polarRoot[i].modu) > 1.0)) then
            begin
              Sysbeep(3);
              Writeln('    The modulus of the root must be less than unity');
            end;
        until (polarRoot[i].modu < 1) or (stability = false);
        if i <= 2 * r then
          i := i + 2
        else
          i := i + 1;
      end; {while}

    Writeln;
    for i := 1 to p do
      begin
        Write('    rho(', i : 1, ') = ', polarRoot[i].modu : 8 : 4);
        Writeln('    omega(', i : 1, ') = ', polarRoot[i].argu : 8 : 4);
      end;
    Writeln;

    Writeln('    Press <RETURN> to continue');
    Readln;

{CONVERSION FROM POLAR TO CARTESIAN: DISCRETE CASE}
    if (discontent = discrete) then
      begin
        for i := 1 to p do
          begin {i}
            if Abs(polarRoot[i].argu) < virtualZero then
              begin {if}
                cartRoot[i].re := polarRoot[i].modu;
                cartRoot[i].im := 0.0;
              end {if}
            else
              begin {else}
                theta := polarRoot[i].argu * degToRad;
                rho := polarRoot[i].modu;
                cartRoot[i].re := rho * Cos(theta);
                cartRoot[i].im := rho * Sin(theta);
              end; {else}
          end; {i}
      end; {discontent = discrete }

{CONVERSION FROM POLAR TO CARTESIAN: CONTINUOUS CASE}
    if (discontent = continuous) then
      begin
        for i := 1 to p do
          begin {i}
          {CONVERT THE MODULUS}
            cartRoot[i].re := Ln(polarRoot[i].modu);

           {CONVERT THE ARGUMENT}
            if (Abs(polarRoot[i].argu) < virtualZero) then
              cartRoot[i].im := 0.0
            else if (polarRoot[i].argu = 180) then
              begin {else if}
                cartRoot[i].im := 0.0;
                cartRoot[i].re := -cartRoot[i].re;
              end {else if}
            else
              cartRoot[i].im := polarRoot[i].argu * degToRad;
          end; {i}
      end; {discontent = continuous}

    for i := 1 to p do
      begin
        Write('    root(', i : 1, ').re = ', cartRoot[i].re : 8 : 4);
        Writeln('    root(', i : 1, ').im = ', cartRoot[i].im : 8 : 4);
      end;

    Writeln;
    Writeln('    Press <RETURN> to continue');
    Readln;


  end; {PolarAcquisition}

 {_________________________________________________}

  procedure CoeffAcquisition (var g: integer;
                  var gamma: complexVector;
                  var coeffStatus: integer;
                  stability: boolean);

    var
      i: integer;
      stable: boolean;
      alpha: vector;

    label
      99;

  begin
    goto 99;
    if discontent = discrete then
      begin
        ClrScr;
        Writeln;
        Writeln;
        Writeln('    THE COEFFICIENTS OF A DISCRETE-TIME MODEL');
        Writeln;
        Writeln('    The coefficient alpha_p of a discrete-time polynomial lag operator');
        Writeln('    is indexed by an integer that takes the same value as the power of');
        Writeln('    the  associated operator L^p. Thus, the pth term of the polynomial');
        Writeln('    is  alpha_p L^p.');
        Writeln(' ' : 62, '<RETURN>');
        Readln;
        stability := true;
      end
    else
      begin
        ClrScr;
        Writeln;
        Writeln;
        Writeln('    THE COEFFICIENTS OF A CONTINUOUS-TIME MODEL');
        Writeln;
        Writeln('    The coefficients of the derivatives within the continuous-time');
        Writeln('    polynomial operator are indexed in reverse order of the powers');
        Writeln('    of  the derivatives. Thus, within  the polynomial of degree p,');
        Writeln('    the jth term is  phi_j D^{p - j}.');
        Writeln(' ' : 59, '<RETURN>');
        Readln;
        stability := false;
      end;
99:
    Writeln;
    Writeln('    Real coefficients');

    repeat
      Writeln;
      for i := 0 to g do
        begin
          Writeln('    Specify   coeff[', i : 1, ']');
          ReadANumber(gamma[i].re);
          gamma[i].im := 0;
          alpha[i] := gamma[i].re;
        end;

      if (discontent = continuous) then
        begin
          RouthCriterion(alpha, g, stable);
          Writeln;
          if stable then
            Writeln('    The system is stable')
          else
            begin
              Sysbeep(3);
              Writeln('    The system is unstable')
            end;
          Readln;
        end;

      if stability and (discontent = discrete) then
        begin {test for stability}
          JuryCriterion(alpha, g, stable);
          if not stable then
            begin
              SysBeep(3);
              Writeln('    The roots are unstable!');
              Writeln('    Respecify the coefficients.');
            end;
        end; {test for stability}
    until (stable or (not stability));

{FindRoots is included in ParameterAcquisition}

  end; {CoeffAcquisition}

 {_________________________________________________}

  procedure ParameterAcquisition (var p, r: integer;
                  var alpha: shortVector;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

    const
      almostZero = 0.00001;

{This procedure mediates thee different ways of specifing the prameters via the}
{polar forms of the roots, their caresian forms, or via the polynomial coefficients}
{We might replace the real-valued coefficient vector by a complex vector}

    var
      i, coeffStatus: integer;
      polarSpec, cartSpec, coeffSpec: boolean;
      complexAlpha: complexVector;

  begin
    complexAlpha[0].re := 1;

    polarSpec := false;
    cartSpec := false;
    coeffSpec := false;

{if discontent = discrete then}

    YesOrNo('    Do you want to specify the auxiliary polynomial in polar form,', polarSpec);
    if polarSpec = false then
      YesOrNo('    Do you want to specify in Cartesian form,', cartSpec);
    if (polarSpec = false) and (cartSpec = false) then
      begin
        coeffSpec := true;
        Writeln('    You have chosen to specify the polynomial coefficients');
      end;

    if polarSpec then
      PolarAcquisition(p, r, cartRoot, polarRoot, stability)
    else if cartSpec then
      RootsAcquisition(p, r, cartRoot, polarRoot, stability)
    else if coeffSpec then
      CoeffAcquisition(p, complexAlpha, coeffStatus, stability);

    if coeffSpec then
      begin
        FindRoots(p, complexAlpha, cartRoot, polarRoot);
        Writeln;
        for i := 1 to p do
          begin {i}
            Write('    root(', i : 1, ').re = ', cartRoot[i].re : 8 : 4);
            Writeln('    root(', i : 1, ').im = ', cartRoot[i].im : 8 : 4);
            if (Abs(cartRoot[i].im) < almostZero) then
              cartRoot[i].im := 0.0;
          end; {i}
        Writeln;
      end
    else {if polarSpec of cartSpec}
      InverseRootsToCoeffs(p, cartRoot, complexAlpha);

    for i := 0 to p do
      alpha[i] := complexAlpha[i].re;

  end; {ParameterAcquisition}

 {_________________________________________________}

 procedure MAverageAcquisition (var p, r: integer;
                  var alpha: shortVector;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

{The pupose of this routine is to suspend the various options for parameter}
{acquistion in the case of a MA(1) component when CoeffAcquisition is used}

    const
      almostZero = 0.00001;

    var
      i, coeffStatus: integer;
      complexAlpha: complexVector;

  begin
    complexAlpha[0].re := 1;

    Writeln('    You have chosen to specify the polynomial coefficients');
    CoeffAcquisition(p, complexAlpha, coeffStatus, stability);

    FindRoots(p, complexAlpha, cartRoot, polarRoot);
    Writeln;
    for i := 1 to p do
      begin {i}
        Write('    root(', i : 1, ').re = ', cartRoot[i].re : 8 : 4);
        Writeln('    root(', i : 1, ').im = ', cartRoot[i].im : 8 : 4);
        if (Abs(cartRoot[i].im) < almostZero) then
          cartRoot[i].im := 0.0;
      end; {i}
    Writeln;


    for i := 0 to p do
      alpha[i] := complexAlpha[i].re;

  end; {MAverageAcquisition}

 {_________________________________________________}

  procedure ARMAacquisition (var ARMASpec: ARMARecord;
                  stability: boolean);

{This is a modified version of ARMAacquisition specific to the ARMA(2, 1) case}

    var
      i: integer;

  begin
    ClrScr;

    Writeln;
    Writeln('    SPECIFY A DISCRETE-TIME AUTOREGRESSIVE MOVING-AVERAGE MODEL');
    Writeln;
    Writeln('    You are constrained, in this  version of the program to choose an');
    Writeln('    ARMA(2, 1) model with conjugate complex poles, i.e. autoregessive');
    Writeln('    roots. For stability, these must be inside the unit circle.');
    Writeln;
    Writeln(' ' : 61, '<RETURN>');
    Readln;

    stability := true;

    Writeln('    Specify the Autoregressive Parameters');
    Writeln;

    ARMASpec.p := 2;
    ARMASpec.pr := 1;
    ParameterAcquisition(ARMASpec.p, ARMASpec.pr, ARMASpec.alpha, ARMASpec.poles, ARMASpec.polarPoles, stability);
    Writeln;
    stability := false;

    Writeln('    Specify the Moving-Average Parameters');
    Writeln;

    ARMASpec.q := 1;
    ARMASpec.qr := 0;
    MAverageAcquisition(ARMASpec.q, ARMASpec.qr, ARMASpec.beta, ARMASpec.zeros, ARMASpec.polarZeros, stability);

    if (ARMASpec.p > 0) then
      begin
        Writeln('    Denominator Coefficients');
        Writeln;
        for i := 0 to ARMASpec.p do
          Writeln('    alpha(', i : 1, ') = ', ARMASpec.alpha[i] : 8 : 4);

        Writeln;
        Writeln('    Denominator Cartesian Roots');
        Writeln;
        for i := 1 to ARMASpec.p do
          Writeln('    root[', i : 1, '].re = ', ARMASpec.poles[i].re : 8 : 4, '    root[', i : 1, '].im = ', ARMASpec.poles[i].im : 8 : 4);
        Writeln;
        Writeln('    Denominator Polar Roots');
        Writeln;
        for i := 1 to ARMASpec.p do
          Writeln('    polar[', i : 1, '].modu = ', ARMASpec.PolarPoles[i].modu : 8 : 4, '    pole[', i : 1, '].argu = ', ARMASpec.PolarPoles[i].argu : 8 : 4);

        Writeln;
        Writeln('    Press <RETURN> to continue');
        Readln;
      end; {ARMASpec.p > 0}

    if (ARMASpec.q > 0) then
      begin
        Writeln;
        Writeln('    Numerator Coefficients');
        Writeln;
        for i := 0 to ARMASpec.q do
          Writeln('    beta(', i : 1, ') = ', ARMASpec.beta[i] : 8 : 4);

        Writeln;
        Writeln('    Numerator Cartesian Roots');
        Writeln;
        for i := 1 to ARMASpec.q do
          Writeln('    root[', i : 1, '].re = ', ARMASpec.zeros[i].re : 8 : 4, '    root[', i : 1, '].im = ', ARMASpec.zeros[i].im : 8 : 4);
        Writeln;
        Writeln('    Numerator Polar Roots');
        Writeln;
        for i := 1 to ARMASpec.q do
          Writeln('    zero[', i : 1, '].modu = ', ARMASpec.PolarZeros[i].modu : 8 : 4, '    zero[', i : 1, '].argu = ', ARMASpec.PolarZeros[i].argu : 8 : 4);

        Writeln;
        Writeln('    Press <RETURN> to continue');
        Readln;
      end; {ARMASpec.q > 0}

    ARMASpec.noModel := false;
    ARMASpec.rootsFound := true;
    ARMASpec.parametersFound := true;
    ARMASpec.varEpsilon := 1.0;

    LSDESpec.noModel := true;
    CARMASpec.noModel := true;
    EDLMSpec.noModel := true;

    ClrScr;

  end; {ARMAacquisition}

 {_________________________________________________}

  procedure altARMAacquisition (var ARMASpec: ARMARecord;
                  stability: boolean);

 {This is a modified version of ARMAacquisition specific to the ARMA(2, 1) case}

    var
      i: integer;

  begin
    ClrScr;

    Writeln;
    Writeln('    SPECIFY A DISCRETE-TIME AUTOREGRESSIVE MOVING-AVERAGE MODEL');
    Writeln;
    Writeln('    You are constrained, in this version of the program, to choose an');
    Writeln('    ARMA(2, 1) model with conjugate complex poles, i.e. autoregessive');
    Writeln('    roots. For stability, these must be inside the unit circle.');
    Writeln;
    Writeln(' ' : 61, '<RETURN>');
    Readln;

    stability := true;
  {This works  for  ARMASpec and for CARMASpec indirectly}

    Writeln('    Specify the Autoregressive Parameters');
    Writeln;

    ARMASpec.p := 2;
    ARMASpec.pr := 1;
    ParameterAcquisition(ARMASpec.p, ARMASpec.pr, ARMASpec.alpha, ARMASpec.poles, ARMASpec.polarPoles, stability);
    Writeln;
    stability := false;

    Writeln('    Specify the Moving-Average Parameters');
    Writeln;

    ARMASpec.q := 1;
    ARMASpec.qr := 0;
    MAverageAcquisition(ARMASpec.q, ARMASpec.qr, ARMASpec.beta, ARMASpec.zeros, ARMASpec.polarZeros, stability);

    if (ARMASpec.p > 0) then
      begin
        Writeln('    Denominator Coefficients');
        Writeln;
        for i := 0 to ARMASpec.p do
          Writeln('    alpha(', i : 1, ') = ', ARMASpec.alpha[i] : 8 : 4);

        Writeln;
        Writeln('    Denominator Cartesian Roots');
        Writeln;
        for i := 1 to ARMASpec.p do
          Writeln('    root[', i : 1, '].re = ', ARMASpec.poles[i].re : 8 : 4, '    root[', i : 1, '].im = ', ARMASpec.poles[i].im : 8 : 4);
        Writeln;
        Writeln('    Denominator Polar Roots');
        Writeln;
        for i := 1 to ARMASpec.p do
          Writeln('    polar[', i : 1, '].modu = ', ARMASpec.PolarPoles[i].modu : 8 : 4, '    pole[', i : 1, '].argu = ', ARMASpec.PolarPoles[i].argu : 8 : 4);

        Writeln;
        Writeln('    Press <RETURN> to continue');
        Readln;
      end; {ARMASpec.p > 0}

    if (ARMASpec.q > 0) then
      begin
        Writeln;
        Writeln('    Numerator Coefficients');
        Writeln;
        for i := 0 to ARMASpec.q do
          Writeln('    beta(', i : 1, ') = ', ARMASpec.beta[i] : 8 : 4);

        Writeln;
        Writeln('    Numerator Cartesian Roots');
        Writeln;
        for i := 1 to ARMASpec.q do
          Writeln('    root[', i : 1, '].re = ', ARMASpec.zeros[i].re : 8 : 4, '    root[', i : 1, '].im = ', ARMASpec.zeros[i].im : 8 : 4);
        Writeln;
        Writeln('    Numerator Polar Roots');
        Writeln;
        for i := 1 to ARMASpec.q do
          Writeln('    zero[', i : 1, '].modu = ', ARMASpec.PolarZeros[i].modu : 8 : 4, '    zero[', i : 1, '].argu = ', ARMASpec.PolarZeros[i].argu : 8 : 4);

        Writeln;
        Writeln('    Press <RETURN> to continue');
        Readln;
      end; {ARMASpec.q > 0}

    ARMASpec.noModel := false;
    ARMASpec.rootsFound := true;
    ARMASpec.parametersFound := true;
    ARMASpec.varEpsilon := 1.0;

    LSDESpec.noModel := true;
    CARMASpec.noModel := true;
    EDLMSpec.noModel := true;

    ClrScr;

  end; {altARMAacquisition}

{_________________________________________________}

  procedure LSDEAcquisition (var LSDESpec: ARMARecord;
                  stability: boolean);

 {This works equally for  LSDESpec and CARMASpec }
 {N.B. we should declare the orders p and q within this segment}
 {and we should impose the restriction p > q.}

    var
      i: integer;
      poleQ, zeroQ: boolean;

  begin
    ClrScr;
    Writeln;
    Writeln('       SPECIFY A LINEAR STOCHASTIC DIFFERENTIAL EQUATION MODEL');
    Writeln;
    Writeln('       The factors of a continuous-time polynomial derivatives operator');
    Writeln('       take the form of (D - kappa).');
    Writeln;
    Writeln('       If kappa = delta + i omega = ln(rho) + i omega is a complex pole,');
    Writeln('       then it  can be  specified in  terms of its  Cartesian components');
    Writeln('       delta and  omega  or in  polar form, via the  modulus rho and the');
    Writeln('       argument omega, expressed in degrees.');
    Writeln;
    Writeln('       Observe that, for stability, delta must  be a negative number and');
    Writeln('       that, equivalently, rho must lie in the interval [0, 1).');
    Writeln;
    Writeln('       The order p of the denominator polynomial of the transfer function,');
    Writeln('       which is the  autoregressive part, must  exceed  the order q of the');
    Writeln('       numerator polynomial, which is the moving-average part.');
    Writeln;
    Writeln(' ' : 66, '<RETURN>');
    Readln;
    stability := true;

    YesOrNo('    Are there any poles', poleQ);
    if poleQ then
      begin
        Writeln('    What is the order of the denominator polynomial?');
        GetBoundedInteger(0, 10, LSDESpec.p);
        ParameterAcquisition(LSDESpec.p, LSDESpec.pr, LSDESpec.alpha, LSDESpec.poles, LSDESpec.polarPoles, stability)
      end
    else
      LSDESpec.alpha[0] := 1.0;
    Writeln;
    stability := false;

    YesOrNo('    Are there any zeros', zeroQ);
    if zeroQ then
      begin
        Writeln('    What is the order of the numerator polynomial?');
        Writeln('    (This must be less that the denominator order)');
        GetBoundedInteger(0, LSDESpec.p - 1, LSDESpec.q);
        ParameterAcquisition(LSDESpec.q, LSDESpec.qr, LSDESpec.beta, LSDESpec.zeros, LSDESpec.polarZeros, stability)
      end
    else
      LSDESpec.beta[0] := 1.0;

    {Print out the acquired coefficients}
    if (LSDESpec.p > 0) then
      begin
        Writeln('    Denominator Coefficients');
        Writeln;
        for i := 0 to LSDESpec.p do
          Writeln('    phi(', i : 1, ') = ', LSDESpec.alpha[i] : 8 : 4);

        Writeln;
        Writeln('    Denominator Cartesian Roots');
        Writeln;
        for i := 1 to LSDESpec.p do
          begin
            Write('    root[', i : 1, '].re = ', LSDESpec.poles[i].re : 8 : 4);
            Writeln('    root[', i : 1, '].im = ', LSDESpec.poles[i].im : 8 : 4);
          end;

        Writeln;
        Writeln('    Denominator Polar Roots');
        Writeln;
        for i := 1 to LSDESpec.p do
          begin
            Write('    polar[', i : 1, '].modu = ', LSDESpec.PolarPoles[i].modu : 8 : 4);
            Writeln('    pole[', i : 1, '].argu = ', LSDESpec.PolarPoles[i].argu : 8 : 4);
          end;

        Writeln;
        Writeln('    Press <RETURN> to continue');
        Readln;
      end; {LSDESpec.p > 0}

    if (LSDESpec.q > 0) then
      begin
        Writeln;
        Writeln('    Numerator Coefficients');
        Writeln;
        for i := 0 to LSDESpec.q do
          Writeln('    theta(', i : 1, ') = ', LSDESpec.beta[i] : 8 : 4);

        Writeln;
        Writeln('    Numerator Cartesian Roots');
        Writeln;
        for i := 1 to LSDESpec.q do
          begin
            Write('    root[', i : 1, '].re = ', LSDESpec.zeros[i].re : 8 : 4);
            Writeln('    root[', i : 1, '].im = ', LSDESpec.zeros[i].im : 8 : 4);
          end;
        Writeln;
        Writeln('    Numerator Polar Roots');
        Writeln;
        for i := 1 to LSDESpec.q do
          begin
            Writeln('    zero[', i : 1, '].modu = ', LSDESpec.PolarZeros[i].modu : 8 : 4);
            Writeln('    zero[', i : 1, '].argu = ', LSDESpec.PolarZeros[i].argu : 8 : 4);
          end;
        Writeln;
        Writeln('    Press <RETURN> to continue');
        Readln;
      end; {LSDESpec.q > 0}

    LSDESpec.noModel := false;
    LSDESpec.rootsFound := true;
    LSDESpec.parametersFound := true;
    LSDESpec.varEpsilon := 1.0;

    ARMASpec.noModel := true;
    CARMASpec.noModel := true;
    EDLMSpec.noModel := true;

    ClrScr;

  end; {LSDEAcquisition}

 {_________________________________________________}

  procedure GetLags (var lags: integer);

  begin
    Writeln('    How Many Lagged AutoCovariances?');
    repeat
      ReadASmallInteger(lags);
      if lags > 20 then
        begin
          SysBeep(3);
          Writeln('    The maximum number of autocovariances is 20!');
          Writeln;
        end;
    until lags <= 20;
    Writeln;
  end; {GetLags}

 {_________________________________________________}

end. {ACQUIRE: Unit 13 of CONCRETE.PAS}

