unit LEGENDS; {Unit 4 of CONCRETE.PAS}

interface

  uses
    GLOBALS, UTILS, Graph, Dos, Crt;

  function ARMAString (var ARMASpec: ARMARecord): string;

  function LSDEString (var LSDESpec: ARMARecord): string;

  function ARMASpectLegend (var ARMASpec: ARMARecord): string;

  function EDLMcovarLegend (var EDLMSpec: ARMARecord): string;

  function LSDESpectLegend (var ARMASpec: ARMARecord): string;

  function OverLaySpectLegend (var LSDESpec, ARMASpec: ARMARecord): string;

  function ARMAcovarLegend (var ARMASpec: ARMARecord): string;

  function LSDEAcovarLegend (var LSDESpec: ARMARecord): string;

  procedure WriteARParameters (var ARMASpec: ARMARecord);

  procedure WriteMAParameters (var ARMASpec: ARMARecord);

  procedure WritePhiParameters (var LSDESpec: ARMARecord);

  procedure WriteThetaParameters (var LSDESpec: ARMARecord);

  procedure WritePolarPoles (var ARMASpec: ARMARecord);

  procedure WritePolarZeros (var ARMASpec: ARMARecord);

  procedure WriteCartPoles (var ARMASpec: ARMARecord);

  procedure WriteCartZeros (var ARMASpec: ARMARecord);

  procedure OpenARMAFile (var F: text;
                  var fileIsOpen: boolean);

  procedure OpenLSDEFile (var F: text;
                  var fileIsOpen: boolean);

  procedure SaveARParameters (var F: text;
                  var ARMASpec: ARMARecord);

  procedure SaveMAParameters (var F: text;
                  var ARMASpec: ARMARecord);

  procedure SavePhiParameters (var F: text;
                  var LSDESpec: ARMARecord);

  procedure SaveThetaParameters (var F: text;
                  var LSDESpec: ARMARecord);

  procedure SavePolarPoles (var F: text;
                  var ARMASpec: ARMARecord);

  procedure SavePolarZeros (var F: text;
                  var ARMASpec: ARMARecord);

  procedure SaveCartPoles (var F: text;
                  var ARMASpec: ARMARecord);

  procedure SaveCartZeros (var F: text;
                  var ARMASpec: ARMARecord);

implementation

 {_________________________________________________}

  function Sgn (number: real): char;
  begin
    if number >= 0 then
      Sgn := '+'
    else
      Sgn := ' ';
  end; {Sgn}

{_________________________________________________}

  function ARMAString (var ARMASpec: ARMARecord): string;

    var
      legend, numberString, pString, qString: string;
      i: integer;

  begin
    with ARMASpec do
      begin {with ARMASpec}
        pString := IntegerToString(p);
        qString := IntegerToString(q);

        if (p = 0) and (q = 0) then
          legend := ' white-noise process';
        if (p = 0) and (q > 0) then
          legend := Concat('an MA(', qString, ') process ');
        if (q = 0) and (p > 0) then
          Write(legend, 'a AR(', p : 1, ') process ');
        legend := Concat('an AR(', pString, ') process ');
        if (p > 0) and (q > 0) then
          legend := Concat('an ARMA(', pString, ',', qString, ') process ');

        if p > 0 then
          begin

            numberString := TalyToString(alpha[1]);
            if (alpha[1] > 0) then
              numberString := Concat('+', numberString);
            legend := Concat(legend, '(1.0', numberString, 'L');
            for i := 2 to p do
              begin
                pString := IntegerToString(i);
                numberString := TalyToString(alpha[i]);
                if (alpha[i] > 0) then
                  numberString := Concat('+', numberString);
                legend := Concat(legend, numberString, 'L^', pString);
              end;
            legend := Concat(legend, ')y(t) = ');
          end
        else {p=0}
          legend := 'y(t) = ';

        if q > 0 then
          begin
            numberString := TalyToString(beta[1]);
            if (beta[1] > 0) then
              numberString := Concat('+', numberString);
            legend := Concat(legend, '(1', numberString, 'L');
            for i := 2 to q do
              begin
                qString := IntegerToString(i);
                numberString := TalyToString(beta[i]);
                if (beta[i] > 0) then
                  numberString := Concat('+', numberString);
                legend := Concat(legend, numberString, 'L^', qString);
              end;
            legend := Concat(legend, ')e(t)');
          end
        else {q=0}
          legend := Concat(legend, 'e(t)');
        ARMAString := legend;
      end; {with ARMASpec}

  end; {ARMAString}

 {_________________________________________________}

  function LSDEString (var LSDESpec: ARMARecord): string;

    var
      legend, numberString, pString, qString: string;
      i: integer;

  begin

    with LSDESpec do
      begin
        pString := IntegerToString(p);
        qString := IntegerToString(q);

        if (p > 0) and (q >= 0) then
          legend := Concat('an LSDE(', pString, ',', qString, ') process ');

        if p > 0 then
          begin
            if p > 1 then
              legend := Concat(legend, '(D^', pString)
            else
              legend := Concat(legend, '(D');
            for i := 1 to p - 1 do
              begin
                pString := IntegerToString(p - i);
                numberString := TalyToString(alpha[i]);
                if (alpha[i] > 0) then
                  numberString := Concat('+', numberString);
                if (i > 1) then
                  legend := Concat(legend, numberString, 'D^', pString)
                else
                  legend := Concat(legend, numberString, 'D')
              end;
            numberString := TalyToString(alpha[p]);
            if (alpha[p] > 0) then
              numberString := Concat('+', numberString);
            legend := Concat(legend, numberString, ')y(t) = ');
          end
        else {p=0}
          legend := 'y(t) = ';

        if (q > 0) then
          begin
            numberString := TalyToString(beta[0]);
            if (q > 1) then
              legend := Concat(legend, '(', numberString, 'D^', qString)
            else
              legend := Concat(legend, '(', numberString, 'D');
            for i := 1 to q - 1 do
              begin
                qString := IntegerToString(q - i);
                numberString := TalyToString(beta[i]);
                if (beta[i] > 0) then
                  numberString := Concat('+', numberString);
                if (i > 1) then
                  legend := Concat(legend, numberString, 'D^', pString)
                else
                  legend := Concat(legend, numberString, 'D')
              end;
            numberString := TalyToString(beta[q]);
            if (beta[q] > 0) then
              numberString := Concat('+', numberString);
            legend := Concat(legend, numberString, ')z(t)');
          end
        else {q=0}
          legend := 'z(t)';

        LSDEString := legend;

      end; {with LSDESpec}

  end; {LSDEString}

 {_________________________________________________}

  function ARMASpectLegend (var ARMASpec: ARMARecord): string;

    var
      legend, addString: string;

  begin

    legend := ' The spectrum of ';
    addString := ARMAString(ARMASpec);
    legend := Concat(legend, addString);

    ARMASpectLegend := legend;

  end; {ARMASpectLegend}

 {_________________________________________________}

  function LSDESpectLegend (var ARMASpec: ARMARecord): string;

    var
      legend, addString: string;

  begin

    legend := ' The spectral density function of ';
    addString := LSDEString(LSDESpec);
    legend := Concat(legend, addString);

    LSDESpectLegend := legend;

  end; {LSDESpectLegend}

 {_________________________________________________}

  function OverLaySpectLegend (var LSDESpec, ARMASpec: ARMARecord): string;

    var
      legend, addString: string;

  begin
    legend := ' The spectrum of ';
    addString := ARMAString(ARMASpec);
    legend := Concat(legend, addString);
    addString := ' superimposed on the spectrum of ';
    legend := Concat(legend, addString);
    addString := LSDEString(LSDESpec);
    legend := Concat(legend, addString);

    OverLaySpectLegend := legend;

  end; {OverLaySpectLegend}

 {_________________________________________________}

  function ARMAcovarLegend (var ARMASpec: ARMARecord): string;

    var
      legend, addString: string;

  begin

    with ARMASpec do
      begin {with ARMASpec}
        legend := ' The theoretical autocorrelation function of ';
        addString := ARMAString(ARMASpec);
        legend := Concat(legend, addString);

      end; {with ARMASpec}

    ARMAcovarLegend := legend;

  end; {ARMAcovarLegend}

 {_________________________________________________}

  function EDLMcovarLegend (var EDLMSpec: ARMARecord): string;

    var
      legend, addString: string;

  begin

    with EDLMSpec do
      begin {with EDLMSpec}
        legend := ' The autocorrelation function of the EDLM ';
        addString := ARMAString(EDLMSpec);
        legend := Concat(legend, addString);

      end; {with EDLMSpec}

    EDLMcovarLegend := legend;

  end; { EDLMcovarLegend}
{_________________________________________________}

  function LSDEAcovarLegend (var LSDESpec: ARMARecord): string;

    var
      legend, addString: string;

  begin

    with LSDESpec do
      begin {with ARMASpec}
        legend := ' The theoretical autocorrelation function of ';
        addString := LSDEString(LSDESpec);
        legend := Concat(legend, addString);

      end; {with LSDESpec}

    LSDEAcovarLegend := legend;

  end; {LSDEAcovarLegend}

 {_________________________________________________}


  procedure WriteARParameters (var ARMASpec: ARMARecord);

    var
      j: integer;

  begin

    with ARMASpec do
      begin {ARMASpec}

        if (p > 0) then
          begin {p > 0}
            Writeln;
            Writeln('    The Autoregressive Parameters');
            Writeln;
            for j := 0 to p do
              Writeln('      alpha[', j : 1, '] = ', alpha[j] : 6 : 4);
            Writeln;
          end; {p > 0}

      end; {ARMASpec}

  end; {WriteARParameters}

{________________________________________________}

  procedure WriteMAParameters (var ARMASpec: ARMARecord);

    var
      j: integer;

  begin

    with ARMASpec do
      begin {ARMASpec}

        if (q > 0) then
          begin {q > 0}
{Writeln;}
            Writeln('    The Moving-Average Parameters');
            Writeln;
            for j := 0 to q do
              Writeln('      beta[', j : 1, '] = ', beta[j] : 8 : 4);
            Writeln;
          end;  {q > 0}

      end; {ARMASpec}

  end; {WriteMAParameters}

{________________________________________________}

  procedure WritePhiParameters (var LSDESpec: ARMARecord);

    var
      j: integer;

  begin

    with LSDESpec do
      begin {LSDESpec}

        if (p > 0) then
          begin {p > 0}
            Writeln;
            Writeln('    The Autoregressive Parameters');
            Writeln;
            for j := 0 to p do
              Writeln('      phi[', j : 1, '] = ', alpha[j] : 6 : 4);
            Writeln;
          end; {p >0}

      end; {LSDESpec}

  end; {WritePhiParameters}

{________________________________________________}

  procedure WriteThetaParameters (var LSDESpec: ARMARecord);

    var
      j: integer;

  begin

    with LSDESpec do
      begin {LSDESpec}

        if (q > 0) then
          begin {q > 0}
            Writeln;
            Writeln('    The Moving-Average Parameters');
            Writeln;
            for j := 0 to q do
              Writeln('      theta[', j : 1, '] = ', beta[j] : 6 : 4);
            Writeln;
          end; {q > 0}

      end; {LSDESpec}

  end; {WriteLSDEParameters}

{________________________________________________}

  procedure WritePolarPoles (var ARMASpec: ARMARecord);

    var
      j: integer;

  begin
    with ARMASpec do
      begin {ARMASpec}

        if (p > 0) then
          begin {p > 0}
            Writeln;
            Writeln('    The Autoregressive Roots: Polar Form');
            Writeln;
            for j := 1 to p do
              begin {j}
                Write('    pole[', j : 1, '].modu = ', polarPoles[j].modu : 8 : 4);
                Writeln('    pole[', j : 1, '].argu = ', polarPoles[j].argu : 8 : 2);
              end; {j}
          end; {p > 0}

      end; {ARMASpec}

  end; {WritePolarPoles}

{________________________________________________}

  procedure WritePolarZeros (var ARMASpec: ARMARecord);

    var
      j: integer;

  begin
    with ARMASpec do
      begin {ARMASpec}

        if (q > 0) then
          begin {q > 0 }
            Writeln;
            Writeln('    The Moving-Average Roots: Polar Form');
            Writeln;
            for j := 1 to q do
              begin {j}
                Write('    zero[', j : 1, '].modu = ', polarZeros[j].modu : 8 : 4);
                Writeln('    zero[', j : 1, '].argu = ', polarZeros[j].argu : 8 : 2);
              end; {j}
          end; {q > 0}

      end; {ARMASpec}

  end; {WritePolarZeros}

{________________________________________________}

  procedure WriteCartPoles (var ARMASpec: ARMARecord);

    var
      j: integer;

  begin
    with ARMASpec do
      begin {ARMASpec}

        if (p > 0) then
          begin {p > 0}
            Writeln;
            Writeln('    The Autoregressive Roots: Cartesian Form');
            Writeln;
            for j := 1 to p do
              begin {j}
                Write('    pole[', j : 1, '].re = ', poles[j].re : 8 : 4);
                Writeln('    pole[', j : 1, '].im = ', poles[j].im : 8 : 4);
              end; {j}
          end; {p > 0}

      end; {ARMASpec.p}

  end; {WriteCartPoles}

{________________________________________________}

  procedure WriteCartZeros (var ARMASpec: ARMARecord);

    var
      j: integer;

  begin

    with ARMASpec do
      begin {ARMASpec}

        if (q > 0) then
          begin {q > 0}
            Writeln;
            Writeln('    The Moving-Average Roots: Cartesian Form');
            Writeln;
            for j := 1 to p do
              begin {j}
                Write('    zero[', j : 1, '].re = ', zeros[j].re : 8 : 4);
                Writeln('    zero[', j : 1, '].im = ', zeros[j].im : 8 : 4);
              end; {j}
          end;{q > 0}

      end; {ARMASpec}

  end; {WriteCartZeros}

{________________________________________________}

  procedure OpenARMAFile (var F: text;
                  var fileIsOpen: boolean);

    var
      resultsFileName: string;

  begin

    resultsFileName := 'ARMAparameters.txt';

    fileIsOpen := true;
    Assign(F, resultsFileName);
    ReWrite(F);
    Writeln(F);
    Writeln(F, '    THE PARAMETERS OF THE ARMA MODEL AND OF DERIVED MODELS');
    Writeln(F);


  end; {OpenARMAFile}

{________________________________________________}

  procedure OpenLSDEFile (var F: text;
                  var fileIsOpen: boolean);

    var
      resultsFileName: string;

  begin

    resultsFileName := 'LSDEparameters.txt';
    fileIsOpen := true;
    Assign(F, resultsFileName);
    ReWrite(F);
    Writeln(F);
    Writeln(F);
    Writeln(F, '    THE PARAMETERS OF THE LSDE MODEL AND OF DERIVED MODELS');
    Writeln(F);

  end; {OpenLSDEFile}

{________________________________________________}

  procedure SaveARParameters (var F: text;
                  var ARMASpec: ARMARecord);

    var
      j: integer;

  begin

    with ARMASpec do
      begin {ARMASpec}

        if (p > 0) then
          begin {p > 0}
            Writeln(F);
            Writeln(F, '    The Autoregressive Parameters');
            Writeln(F);
            for j := 0 to p do
              Writeln(F, '      alpha[', j : 1, '] = ', alpha[j] : 6 : 4);
            Writeln(F);
          end; {p > 0}

      end; {ARMASpec}

  end; {SaveARParameters}

{________________________________________________}

  procedure SaveMAParameters (var F: text;
                  var ARMASpec: ARMARecord);

    var
      j: integer;

  begin

    with ARMASpec do
      begin {ARMASpec}

        if (q > 0) then
          begin {q > 0}
{Writeln(F);}
            Writeln(F, '    The Moving-Average Parameters');
            Writeln(F);
            for j := 0 to q do
              Writeln(F, '      beta[', j : 1, '] = ', beta[j] : 8 : 4);
            Writeln(F);
          end;  {q > 0}

      end; {ARMASpec}

  end; {SaveMAParameters}

{________________________________________________}

  procedure SavePhiParameters (var F: text;
                  var LSDESpec: ARMARecord);

    var
      j: integer;

  begin

    with LSDESpec do
      begin {LSDESpec}

        if (p > 0) then
          begin {p > 0}
            Writeln(F);
            Writeln(F, '    The Autoregressive Parameters');
            Writeln(F);
            for j := 0 to p do
              Writeln(F, '      phi[', j : 1, '] = ', alpha[j] : 6 : 4);
            Writeln(F);
          end; {p >0}

      end; {LSDESpec}

  end; {SavePhiParameters}

{________________________________________________}

  procedure SaveThetaParameters (var F: text;
                  var LSDESpec: ARMARecord);

    var
      j: integer;

  begin

    with LSDESpec do
      begin {LSDESpec}

        if (q > 0) then
          begin {q > 0}
{Writeln(F);}
            Writeln(F, '    The Moving-Average Parameters');
            Writeln(F);
            for j := 0 to q do
              Writeln(F, '      theta[', j : 1, '] =', beta[j] : 6 : 4);
            Writeln(F);
          end; {q > 0}

      end; {LSDESpec}

  end; {SaveLSDEParameters}

{________________________________________________}

  procedure SavePolarPoles (var F: text;
                  var ARMASpec: ARMARecord);

    var
      j: integer;

  begin
    with ARMASpec do
      begin {ARMASpec}

        if (p > 0) then
          begin {p > 0}
{Writeln(F);}
            Writeln(F, '    The Autoregressive Roots: Polar Form');
            Writeln(F);
            for j := 1 to p do
              begin {j}
                Write(F, '    pole[ ', j : 1, ' ].modu = ', polarPoles[j].modu : 8 : 4);
                Writeln(F, '    pole[', j : 1, '].argu = ', polarPoles[j].argu : 8 : 2);
              end; {j}
            Writeln(F);
          end; {p > 0}

      end; {ARMASpec}

  end; { SavePolarPoles}

{________________________________________________}

  procedure SavePolarZeros (var F: text;
                  var ARMASpec: ARMARecord);

    var
      j: integer;

  begin
    with ARMASpec do
      begin {ARMASpec}

        if (q > 0) then
          begin {q > 0 }
{Writeln(F);}
            Writeln(F, '    The Moving-Average Roots: Polar Form');
            Writeln(F);
            for j := 1 to q do
              begin {j}
                Write(F, '    zero[ ', j : 1, ' ].modu = ', polarZeros[j].modu : 8 : 4);
                Writeln(F, '    zero[', j : 1, '].argu = ', polarZeros[j].argu : 8 : 2);
              end; {j}
            Writeln(F);
          end; {q > 0}

      end; {ARMASpec}

  end; {SavePolarZeros}

{________________________________________________}

  procedure SaveCartPoles (var F: text;
                  var ARMASpec: ARMARecord);

    var
      j: integer;

  begin
    with ARMASpec do
      begin {ARMASpec}

        if (p > 0) then
          begin {p > 0}
{Writeln(F);}
            Writeln(F, '    The Autoregressive Roots: Cartesian Form');
            Writeln(F);
            for j := 1 to p do
              begin {j}
                Write(F, '    pole[ ', j : 1, ' ].re = ', poles[j].re : 8 : 4);
                Writeln(F, '    pole[', j : 1, '].im = ', poles[j].im : 8 : 4);
              end; {j}
            Writeln(F);
          end; {p > 0}

      end; {ARMASpec.p}

  end; {SaveCartPoles}

{________________________________________________}

  procedure SaveCartZeros (var F: text;
                  var ARMASpec: ARMARecord);

    var
      j: integer;

  begin

    with ARMASpec do
      begin {ARMASpec}

        if (q > 0) then
          begin {q > 0}
{Writeln(F);}
            Writeln(F, '    The Moving-Average Roots: Cartesian Form');
            Writeln(F);
            for j := 1 to p do
              begin {j}
                Write(F, '   zero[ ', j : 1, ' ].re = ', zeros[j].re : 8 : 4);
                Writeln(F, '    zero[', j : 1, '].im = ', zeros[j].im : 8 : 4);
              end; {j}
            Writeln(F);
          end;{q > 0}

      end; {ARMASpec}

  end; {SaveCartZeros}

{________________________________________________}

end. {LEGENDS: Unit 4 of CONCRETE.PAS}

