unit ORGANISE; {Unit 14 of CONCRETE.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, LEGENDS, SCREEN, POST, SPECTRUM, DACOVARS, CACOVARS, ROOTS, CONVERTS, ACQUIRE, Graph, Dos, Crt;

  procedure PageSetUp (var SCREENBOX, POSTBOX: IBOX;
                  var PSPict, TexFormat: boolean);

  procedure GetModelParams (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord);

  procedure DiscreteAndContinuousSpecs (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, cARMASpec, LSDESpec: ARMARecord);

  procedure PlotAutoCovariances (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord);

  procedure MakeTheSpectra (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord);

  procedure TransferStatus (var ARMASpec, CARMASpec, LSDESpec, EDLMSpec: ARMARecord);

  procedure SaveTheResults (var ARMASpec, CARMASpec, LSDESpec, EDLMSpec: ARMARecord);

  procedure SaveAndExit (var ARMASpec, CARMASpec, LSDESpec, EDLMSpec: ARMARecord);

implementation

{_________________________________________________}

  procedure PageSetUp (var SCREENBOX, POSTBOX: IBOX;
                  var PSPict, TexFormat: boolean);
  begin
    ClrScr;

    Writeln;
    Writeln('    SPECIFY THE PAGE PARAMETERS');
    Writeln;

    YesOrNo('    Do you want to output in PostScript', PSPict);
    if PSPict then
      YesOrNo('    Do you want a TeXtures Format', TeXFormat);

    PageParameters(SCREENBOX, POSTBOX);
    noPageParams := False;

  end; {PageSetUp}

{_________________________________________________}

  procedure GetModelParams (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord);

{We don't need  SCREENBOX, or POSTBOX}
    var
      stability, acquireNewModel: boolean;
      DorC: char;

  begin
    ClrScr;

    Writeln;
    Writeln('    SPECIFY THE MODEL PARAMETERS');
    Writeln;

    repeat
      Writeln('    Do you wish to specify a Discrete Model or a Continuous Model, D/C?');
      ReadAChar(DorC);
      DorC := UpCase(DorC);
    until DorC in ['D', 'C'];

    stability := True;
    acquireNewModel := True;

    if (DorC = 'D') then
      begin {discrete}
        discontent := discrete;
        if ARMASpec.noModel = false then
          begin
            SysBeep(3);
            Writeln('    You have already specified a discrete-time model.');
            YesOrNo('    Do you wish to discard it,', acquireNewModel);
          end;
        if acquireNewModel then
          begin
            InitialiseARMARecord(ARMASpec);
            InitialiseARMARecord(LSDESpec);
            InitialiseARMARecord(CARMASpec);
            InitialiseARMARecord(EDLMSpec);
            ARMAacquisition(ARMASpec, stability); {in ACQUIRE}
          end;
      end; {discrete}

    if (DorC = 'C') then
      begin {continuous}
        discontent := continuous;
        if LSDESpec.noModel = False then
          begin
            SysBeep(3);
            Writeln('    You have already specified a continuous model.');
            YesOrNo('    Do you wish to discard it,', acquireNewModel);
          end;
        if acquireNewModel then
          begin
            InitialiseARMARecord(ARMASpec);
            InitialiseARMARecord(LSDESpec);
            InitialiseARMARecord(CARMASpec);
            InitialiseARMARecord(EDLMSpec);
            LSDEacquisition(LSDESpec, stability); {in ACQUIRE}
            EDLMSpec.noModel := true;
          end;
      end; {continuous}

    noModelParams := false;

  end; {GetModelParams}

{_________________________________________________}

  procedure DiscreteAndContinuousSpecs (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, cARMASpec, LSDESpec: ARMARecord);

{This procedure organises the conversions between discrete and continuous forms}

    var
      theMenuItem: integer;
      cNumerator, cDenominator: complexVector;

  begin
    ClrScr;

    if noModelParams then
      begin
        SysBeep(3);
        GetModelParams(SCREENBOX, POSTBOX, ARMASpec, cARMASpec);
      end;

    Writeln;
    Writeln('    DISCRETE AND CONTINUOUS MODELS');
    Writeln;

    if (discontent = discrete) then
      begin
        Writeln('    You have specified a discrete-time ARMA Model.');
        Writeln;
        Writeln('    1.  Display the coefficients of the ARMA model');
        Writeln;
        Writeln('    2.  Display the coefficients of the eqivalent frequency');
        Writeln('        limited continuous-time CARMA model');
        Writeln;
        Writeln('    3.  Calculate and display the coefficients of the corresponding');
        Writeln('        LSDE model driven by a Wiener forcing function');
        Writeln;
        Writeln('                      Select a number and press RETURN');

        GetBoundedInteger(1, 3, theMenuItem);
        ClrScr;

        if (theMenuItem = 1) then
          begin
            Writeln;
            Writeln('    THE COEFFICIENTS  OF THE SPECIFIED ARMA MODEL');
            WriteARParameters(ARMASpec);
            WriteMAParameters(ARMASpec);
          end;

        if (theMenuItem = 2) then
          begin
            if (ARMASpec.q >= ARMASpec.p) then
              begin
                Sysbeep(3);
                Writeln('    The CARMA conversion is available only if the ARMA');
                Writeln('    numerator order p exceeds the denominator order q.');
                Readln;
              end
            else
              begin
                if CARMASpec.noModel then
                  ARMAtoCARMA(ARMASpec, cARMASpec, cNumerator, cDenominator);
                Writeln('    THE COEFFICIENTS OF THE DERIVED CARMA MODEL');
                WritePhiParameters(CARMASpec);
                WriteThetaParameters(CARMASpec);
              end;
          end;

        if (theMenuItem = 3) then
          begin
            if (ARMASpec.q >= ARMASpec.p) then
              begin
                Sysbeep(3);
                Writeln('    The LSDE conversion is available only if the ARMA');
                Writeln('    numerator order p exceeds the denominator order q.');
                Readln;
              end
            else
              begin
                if LSDESpec.noModel then
                  ARMAtoLSDE(ARMASpec, LSDESpec); {in CONVERTS}
                Writeln('    THE COEFFICIENTS OF THE DERIVED LSDE MODEL');
                WritePhiParameters(LSDESpec);
                WriteThetaParameters(LSDESpec);
              end;
          end;

      end; {discontent = discrete}

    if (discontent = continuous) then
      begin
        Writeln('    You have specified a continuous-time LSDE Model');
        Writeln;
        Writeln('    1.  Display the coefficients of the LSDE model');
        Writeln;
        Writeln('    2.  Calculate and display the coefficients of the corresponding');
        Writeln('        Exact Discrete EDLM model');
        Writeln;
        Writeln('                      Select a number and press RETURN');

        GetBoundedInteger(1, 2, theMenuItem);
        ClrScr;

        if (theMenuItem = 1) then
          begin
            Writeln;
            Writeln('    THE COEFFICIENTS OF THE SPECIFIED LSDE MODEL');
            WritePhiParameters(LSDESpec);
            WriteThetaParameters(LSDESpec);
          end;

        if (theMenuItem = 2) then
          begin
            if EDLMSpec.noModel then
              LSDEtoEDLM(LSDESpec, EDLMSpec);  {in CONVERTS}
            Writeln('    THE COEFFICIENTS OF THE DERIVED EDLM MODEL');
            WriteARParameters(EDLMSpec);
            WriteMAParameters(EDLMSpec);
          end;

      end; {discontent = continuous}

    Writeln(' ' : 60, '<RETURN>');
    Readln;

  end; {DiscreteAndContinuousSpecs;}

{_________________________________________________}

  procedure PlotAutoCovariances (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord);

    var
      lags: integer;

  begin
    ClrScr;

    if noModelParams then
      begin
        SysBeep(3);
        GetModelParams(SCREENBOX, POSTBOX, ARMASpec, LSDESpec);
      end;

    if noPageParams then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TexFormat);
      end;

    if discontent = discrete then
      begin {discrete}
        ClrScr;

        Writeln;
        Writeln('      THE AUTOCOVARIANCES OF A DISCRETE-TIME ARMA PROCESS');
        Writeln;
        Writeln('      The  autocovariance function of an MA process is  a finite seqaence');
        Writeln('      whereas that of an AR or  an ARMA process is an infinite  sequence.');
        Writeln;
        Writeln('      When p > q, the  autocovariance function  of  an ARMA(p, q) process');
        Writeln('      is plotted within  a continuous envelope, which also represents the');
        Writeln('      autocovariance function  of a  process driven by a continuous white');
        Writeln('      noise process limited in frequency to the Nyquist interval [0, pi].');
        Writeln;
        Writeln(' ' : 65, '<RETURN>');
        Readln;

        GetLags(lags);
        PlotDiscreteAcovars(SCREENBOX, POSTBOX, ARMASpec, lags); {in DACOVARS}

        if LSDESpec.noModel = false then
          PlotContinuousAcovars(SCREENBOX, POSTBOX, LSDESpec, lags);

      end; {discrete}

    if (discontent = continuous) then
      begin {continuous}
        ClrScr;
        Writeln;
        Writeln('      THE AUTOCOVARIANCES OF THE STOCHASTIC DIFFERENTIAL EQUATION');
        Writeln;
        Writeln('      The  autocovariance function of a linear  stochastic differential');
        Writeln('      equation is a continuous function that is determined by the poles');
        Writeln('      and the zeros of the differential operator.');
        Writeln;
        Writeln('      The autocovariance function is a  weighted sum of real and complex');
        Writeln('      exponential functions  corresponding  to the poles,  which are the');
        Writeln('      roots of the autoregressive component of the operator. The weights');
        Writeln('      are the coefficients of the partial fraction  decomposition of the');
        Writeln('      rational differential operator.');
        Writeln;
        Writeln(' ' : 64, '<RETURN>');
        Readln;
        GetLags(lags);

        PlotContinuousAcovars(SCREENBOX, POSTBOX, LSDESpec, lags);

        if (EDLMSpec.noModel = false) then
          PlotDiscreteAcovars(SCREENBOX, POSTBOX, EDLMSpec, lags); {in DACOVARS}

      end; {continuous}

  end; {PlotAutoCovariances}

{_________________________________________________}

  procedure MakeTheSpectra (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord);

    var
      plotTheEDLM, plotTheLSDE, overlayTheARMA: boolean;
      spectraBOX: rBOX;
      piMultiple: integer;
      pi: real;

  begin
    pi := 4 * ArcTan(1.0);
    ClrScr;

    if noModelParams then
      begin
        SysBeep(3);
        GetModelParams(SCREENBOX, POSTBOX, ARMASpec, LSDESpec);
      end;

    if noPageParams then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TexFormat);
      end;

    if (discontent = discrete) then
      begin
        ClrScr;

        Writeln;
        Writeln('      THE SPECTRUM OF A DISCRETE-TIME ARMA PROCESS');
        Writeln;
        Writeln('      The spectrum of a discrete-time autoregressive moving-average');
        Writeln('      process  is the  cosine Fourier transform  of the sequence of');
        Writeln('      autocovariances. It is  supported on  the  Nyquist  frequency');
        Writeln('      interval  [-pi, pi]. Given that it is symmetric  around zero,');
        Writeln('      it is plotted only over the interval [0, pi].');
        Writeln;
        Writeln(' ' : 62, '<RETURN>');
        Readln;
        PlotSpectrum(SCREENBOX, POSTBOX, ARMASpec);

        if (LSDESpec.noModel = false) then
          begin
            ClrScr;

            Writeln;
            Writeln('      THE SPECTRUM OF THE DERIVED LSDE  MODEL');
            Writeln;
            Writeln('      You may wish to plot the spectral  density function of the Linear');
            Writeln('      Stochastic Differential Equation (LSDE) derived from the discrete');
            Writeln('      autocovariance function of the ARMA model.');
            Writeln;
            YesOrNo('      Do you wish to plot the spectrum of the LSDE,', plotTheLSDE);

            if plotTheLSDE then
              begin {if}
                if LSDESpec.noModel then
                  ARMAtoLSDE(ARMASpec, LSDESpec);
                Writeln;
                Writeln('      Specify the number of radians in the frequency interval as an');
                Writeln('      integer multiple of pi radians, not exceeding 10.');
                GetBoundedInteger(1, 10, piMultiple);
                spectraBOX.xMax := piMultiple * pi;
                Writeln;
                YesOrNo('      Do you wish to superimpose of the ARMA Spectrum,', overlayTheARMA);

                PlotLSDESpectrum(SCREENBOX, POSTBOX, spectraBOX, LSDESpec, ARMASpec, piMultiple, overlayTheARMA);

              end; {if}
          end; {LSDESpec.noModel = false}

      end; {discontent = discrete}

    if (discontent = continuous) then
      begin
        ClrScr;
        Writeln;
        Writeln;
        Writeln('      THE SPECTRUM OF THE STOCHASTIC DIFFERENTIAL EQUATION');
        Writeln;
        Writeln('      The  spectrum of  a stochastic  equation  driven by  a continuous');
        Writeln('      white-noise  forcing  function  that  is  unbounded in  frequency');
        Writeln('      extends over the entire real line. The  spectrum  must be plotted');
        Writeln('      over a specified finite interval including the zero frequency.');
        Writeln;
        Writeln('      The spectrum of a sequence sampled at regular  intervals from the');
        Writeln('      continuous process is supported  on  the interval [-pi, pi]. When');
        Writeln('      plotting the spectral density of the continuous process, there is');
        Writeln('      the option of superimposing the spectrum of the sampled sequence.');
        Writeln;
        Writeln(' ' : 63, '<RETURN>');
        Readln;

        ClrScr;

        Writeln;
        Writeln('      THE SPECTRUM OF THE STOCHASTIC DIFFERENTIAL EQUATION');
        Writeln;
        Writeln('      The spectral density function  will be plotted  over a positive');
        Writeln('      frequency interval running from zero to an upper limit which is');
        Writeln('      an integer multiple of pi radians per sampling interval');
        Writeln;
        Writeln('      Specify the number of radians in the frequency interval as an');
        Writeln('      integer multiple of pi radians, not exceeding 10.');

        GetBoundedInteger(1, 10, piMultiple);
        spectraBOX.xMax := piMultiple * pi;
        overlayTheARMA := false;

        PlotLSDESpectrum(SCREENBOX, POSTBOX, spectraBOX, LSDESpec, ARMASpec, piMultiple, overlayTheARMA);

        if (EDLMSpec.noModel = false) then
          begin
            ClrScr;

            Writeln;
            Writeln('      THE SPECTRUM OF THE DERIVED EXACT DISCRETE LINEAR MODEL');
            Writeln;
            Writeln('      You may wish to plot the spectral density function of the Exact');
            Writeln('      Discrete Linear Model (EDLM) fitted to the sampled ordinates of');
            Writeln('      the continuous autocovariacne function of the Linear Stochastic');
            Writeln('      Differential Equation (LSDE).');
            Writeln;
            YesOrNo('      Do you wish to plot the spectrum of the EDLM,', PlotTheEDLM);

            if PlotTheEDLM then
              begin {if}
                if EDLMSpec.noModel then
                  LSDEtoEDLM(LSDESpec, EDLMSpec);
                PlotSpectrum(SCREENBOX, POSTBOX, EDLMSpec);
              end; {if}
          end; {EDLMSpe.noModel = false}

      end; {discontent = continuous}

  end; {MakeTheSpectra}

{_________________________________________________}

  procedure TransferStatus (var ARMASpec, CARMASpec, LSDESpec, EDLMSpec: ARMARecord);

    var
      ChoiceChar: char;

  begin
    ClrScr;

    Writeln;
    Writeln('    ASSIGN PRIMARY STATUS TO A DERIVED MODEL');
    Writeln;
    Writeln('    The primary model is the parent of the derived models. Primary');
    Writeln('    status may be re-assigned to any available derived model.');
    Writeln;
    Writeln(' ' : 58, '<RETURN>');
    Readln;
    Writeln;

    if not (discontent in [1, 2]) then
      begin
        Sysbeep(3);
        Writeln('   You have not yet specified any models!');
        Writeln(' ' : 58, '<RETURN>');
        Readln;
      end
{--------------------------------------------------}
    else if (discontent = discrete) then

      begin {discrete}
        Writeln('    The status is DISCRETE the primary model is an ARMA model.');
        Writeln;
        if ARMASpec.noModel then
          begin {if}
            Sysbeep(3);
            Writeln('    An ARMA model has not yet been specified.');
            Writeln;
            Writeln(' ' : 58, '<RETURN>');
            Readln;
          end {if}
        else
          begin {else}
            repeat
              if LSDESpec.noModel = false then
                Writeln('    (L)  To make the LSDE the primary model, type L');
              Writeln;
              if CARMASpec.noModel = false then
                begin
                  Writeln('    (C)  To make the CARMA model the Primary model, type C');
                  Writeln;
                end;
              Writeln('    (E)  To return to the main menu, type E');
              ReadLn(choiceChar);
              choiceChar := UpCase(choiceChar)
            until choiceChar in ['L', 'C', 'E'];
          end; {else}

        case choiceChar of
          'L': 
            begin {L}
              if LSDESpec.noModel then
                begin {if}
                  Writeln('    There is no LSDE model.');
                  Writeln;
                  Writeln(' ' : 58, '<RETURN>');
                  Readln
                end {if}
              else
                begin {else}
                  Writeln('    The LSDE model now has primary status.');
                  discontent := continuous;
                  Writeln;
                  Writeln(' ' : 58, '<RETURN>');
                  Readln;
                  {ARMASpec.noModel := true;}
                  {CARMASpec.noModel := true;}
                  InitialiseARMARecord (ARMASpec);
                  InitialiseARMARecord (CARMASpec);
                end; {else}
            end; {L}
          'C': 
            begin {C}
              if CARMASpec.noModel then
                begin {if}
                  Writeln('    There is no CARMA model. None has been derived.');
                  Writeln;
                  Writeln(' ' : 58, '<RETURN>');
                  Readln;
                end {if}
              else
                begin {else}
                  LSDESpec := CARMASpec;
                  Writeln('    The CARMA model now has the primary status as an LSDE.');
                  discontent := continuous;
                  Writeln;
                  Writeln(' ' : 58, '<RETURN>');
                  Readln;
                  {ARMASpec.noModel := true;}
                  {CARMASpec.noModel := true;}
                  InitialiseARMARecord (ARMASpec);
                  InitialiseARMARecord (CARMASpec);
                end; {else}
            end; {C}
          'E': 
            begin
            end;
        end; {case}

      end {discrete}

{--------------------------------------------------}

    else if (discontent = continuous) then

      begin {continuous}
        Writeln('    The status is CONTINUOUS the primary model is an LSDE.');
        Writeln;

        if LSDESpec.noModel then
          begin {if}
            Sysbeep(3);
            Writeln('    An LSDE model has not yet been specified');
            Readln;
          end {if}
        else
          begin {else}
            repeat
              Writeln('    (D)  To make the EDLM the primary model, type D');
              Writeln;
              Writeln('    (E)  To return to the main menu, type E');
              ReadLn(choiceChar);
              choiceChar := UpCase(choiceChar)
            until choiceChar in ['D', 'E'];
          end; {else}

        case choiceChar of
          'D': 
            begin {D}
              if EDLMSpec.noModel then
                begin {if}
                  Sysbeep(3);
                  Writeln('    There is no EDLM model. None has yet been derived.');
                  Writeln;
                  Writeln(' ' : 58, '<RETURN>');
                  Readln
                end {if}
              else
                begin {else}
                  discontent := discrete;
                  ARMASpec := EDLMSpec;
                  Writeln('    The EDLM  now has the status of the ARMA model.');
                  Writeln;
                  Writeln(' ' : 58, '<RETURN>');
                  Readln;
                  {LSDESpec.noModel := true;}
                  InitialiseARMARecord (LSDESpec);
                 end; {else}
            end; {D}
          'E': 
            begin
            end;
        end; {case}

      end; {discontent = continuous}

  end; {TransferStatus}
{_________________________________________________}

  procedure SaveTheResults (var ARMASpec, CARMASpec, LSDESpec, EDLMSpec: ARMARecord);

    var
      theMenuItem: integer;
      fileIsOpen, saveARMAparams, saveEDLMparams: boolean;
      saveCARMAparams, saveLSDEparams, allsDone: boolean;
      F: text;

  begin

    if (discontent = discrete) then
      begin {discrete}
        fileIsOpen := false;

        saveARMAparams := false;
        saveCARMAparams := false;
        saveLSDEparams := false;
        saveEDLMparams := false;
        allsDone := false;

        repeat {until allsDone}
          ClrScr;
;
          Writeln;
          Writeln('    THE PARAMETERS OF THE ARMA MODEL AND OF DERIVED MODELS');
          Writeln;
          Writeln('      1. Save the ARMA Parameters');
          Writeln;
          Writeln('      2. Save the CARMA Parameters');
          Writeln;
          Writeln('      3. Save the LSDE Parameters');
          Writeln;
          Writeln('      4. EXIT and Close the Log File');
          Writeln;
          Writeln('                      Select a number and press RETURN');
          GetBoundedInteger(1, 4, theMenuItem);

          case theMenuItem of
            1: 
              saveARMAparams := true;
            2: 
              begin {2}
                if CARMASpec.noModel then
                  begin {if noModel}
                    Sysbeep(3);
                    Writeln('    There is no CARMA model!');
                    Writeln;
                    if ARMASpec.q >= ARMASpec.p then
                      begin
                        Writeln('    The MA order q exceeds the AR order p. This');
                        Writeln('    prevents you from specfying an CARMA model');
                        Writeln;
                      end;
                    Writeln(' ' : 60, '<RETURN> ');
                    Readln;
                  end {if noModel}
                else
                  saveCARMAparams := true;
              end; {2}
            3: 
              begin {3}
                if LSDESpec.noModel then
                  begin {if noModel}
                    Sysbeep(3);
                    Writeln('    There is no LSDE model!');
                    Writeln;
                    if ARMASpec.q >= ARMASpec.p then
                      begin {q >= p}
                        Writeln('    The MA order q exceeds the AR order p. This');
                        Writeln('    prevents you from specfying an LSDE model');
                        Writeln;
                      end; {q >= p}
                    Writeln(' ' : 60, '<RETURN> ');
                    Readln;
                  end {if noModel}
                else
                  saveLSDEparams := true;
              end; {3}
            4: 
              allsDone := true;
          end; {cases}

        until allsDone;

{ 1.-----------------------------------------------}
        if saveARMAparams then
          begin {ARMA Model}
            if (fileisOpen = false) then
              OpenARMAFile(F, fileisOpen);
            Writeln(F, '    THE PARAMETERS OF THE ARMA MODEL');
            SaveARParameters(F, ARMASpec);
            SaveMAParameters(F, ARMASpec);
            SavePolarPoles(F, ARMASpec);
            SavePolarZeros(F, ARMASpec);
            SaveCartPoles(F, ARMASpec);
            SaveCartZeros(F, ARMASpec);
          end; {ARMA Model}

{ 2.-----------------------------------------------}
        if saveCARMAparams then
          begin {CARMA Model}
            if (FileisOpen = false) then
              OpenARMAFile(F, FileisOpen);
            Writeln(F);
            Writeln(F, '    THE PARAMETERS OF THE DERIVED CARMA MODEL');
            SavePhiParameters(F, CARMASpec);
            SaveThetaParameters(F, CARMASpec);
            SaveCartPoles(F, CARMASpec);
          end; {CARMA Model}

{ 3.-----------------------------------------------}

        if saveLSDEparams then
          begin {LSDE Model}
            if (fileisOpen = false) then
              OpenARMAFile(F, FileisOpen);
            Writeln(F);
            Writeln(F, '    THE PARAMETERS OF THE DERIVED LSDE MODEL');
            SavePhiParameters(F, LSDESpec);
            SaveThetaParameters(F, LSDESpec);
            SaveCartPoles(F, LSDESpec);
          end; {LSDE Model}

        if fileIsOpen then
          Close(F);

      end; {discrete}

{+++++++++++++++++++++++++++++++++++++++++++++++++}
    if (discontent = continuous) then
      begin
        fileIsOpen := false;
        saveLSDEparams := false;
        saveEDLMparams := false;
        allsDone := false;

        repeat {until allsDone}
          ClrScr;

          Writeln;
          Writeln('    THE PARAMETERS OF THE LSDE MODEL AND OF DERIVED MODELS');
          Writeln;
          Writeln('      1. Save the LSDE Parameters');
          Writeln;
          Writeln('      2. Save the EDLM Parameters');
          Writeln;
          Writeln('      3. Close the Log File and RETURN to the Main Menu');
          Writeln;
          Writeln('                      Select a number and press RETURN');
          GetBoundedInteger(1, 3, theMenuItem);

          case theMenuItem of
            1: 
              saveLSDEparams := true;
            2: 
              begin
                if EDLMSpec.noModel then
                  begin {if no EDLM }
                    Sysbeep(3);
                    Writeln('    You have not yet converted the LDSE model to a EDLM');
                    Writeln(' ' : 60, '<RETURN> ');
                    Readln;
                  end {if no EDLM }
                else
                  SaveEDLMparams := true;
              end;
            3: 
              allsDone := true;
          end; {cases}

        until allsDone;

{ 1.-----------------------------------------------}
        if saveLSDEparams then
          begin {LSDE Model}
            if (FileisOpen = false) then
              OpenLSDEFile(F, FileisOpen);
            Writeln(F, '    THE PARAMETERS OF THE LSDE MODEL');
            Writeln;
            SavePhiParameters(F, LSDESpec);
            SaveThetaParameters(F, LSDESpec);
            SaveCartPoles(F, LSDESpec);
          end; {LSDE Model}

{ 2.-----------------------------------------------}

        if saveEDLMparams then
          begin {EDLM Model}
            if fileIsOpen = false then
              OpenARMAFile(F, fileIsOpen);
            Writeln(F);
            Writeln(F, '    THE PARAMETERS OF THE DERIVED EDLM MODEL');
            Writeln;
            SaveARParameters(F, EDLMSpec);
            SaveMAParameters(F, EDLMSpec);
            SavePolarPoles(F, EDLMSpec);
            SaveCartPoles(F, EDLMSpec);
          end; {EDLM Model}


        if fileIsOpen then
          Close(F);

      end; {continuous}

{++++++++++++++++++++++++++++++++++++++++}

  end; {SaveTheResults}

{________________________________________________}

  procedure SaveAndExit (var ARMASpec, CARMASpec, LSDESpec, EDLMSpec: ARMARecord);

    var
      theMenuItem: integer;

  begin
    repeat
      ClrScr;

      Writeln;
      Writeln('    SAVE AND EXIT');
      Writeln;
      Writeln('      1. SAVE Model Parameters');
      Writeln;
      Writeln('      2. RETURN to the Main Menu');
      Writeln;
      Writeln('      3. EXIT the Program');
      Writeln;
      Writeln('                      Select a number and press RETURN');
      GetBoundedInteger(1, 3, theMenuItem);

      case theMenuItem of
        1: 
          SaveTheResults(ARMASpec, CARMASpec, LSDESpec, EDLMSpec);
        2: 
          ; {RETURN to the Main Menu}
        3: 
          allsDone := True; {EXIT the Program}
      end;{case}

    until theMenuItem in [1..3];

  end;{SaveAndExit}

{_________________________________________________}

end. {ORGANISE: Unit 14 of CONCRETE.PAS}

