unit UTILS; {Unit 3 of CONTEXT.PAS}

interface

  uses
    GLOBALS, MATHS, Graph, Dos, Crt;


  procedure InitialiseGraphics (var SCREENBOX: IBOX;
                  var currentMode: integer);

  procedure GetGraphics (var graphicsOn: boolean;
                  var SCREENBOX: IBOX;
                  var currentMode: integer);

  procedure SetTheText;

  procedure RevertText;

  procedure CallForReturn;

  procedure SysBeep (pause: integer);

  procedure ScreenCaption (legend: string);

  procedure MultiCaption (var legend: string);


  procedure PageParameters (var SCREENBOX, POSTBOX: IBOX);

  procedure PageSetUp (var SCREENBOX, POSTBOX: IBOX;
                  var PSPict, TexFormat: Boolean);

  procedure YesOrNo (question: string;
                  var decision: boolean);

  procedure TallyPoints (range: real;
                  var taly: real);

  function UpCase (character: char): char;

  function DownCase (character: char): char;

  procedure StripLeadingBlanks (var theString: string);

  procedure StripTrailingBlanks (var theString: string);

  procedure ReplaceWithin (var theString: string;
                  deletion, replacement: string);

  procedure ReplaceEveryCaseOf (deletion, replacement: string;
                  var theString: string);

  procedure ReadAChar (var theCharacter: char);

  function IntegerToString (theInteger: integer): string;

  function TalyToString (theNumber: real): string;

  function IntegerString (theString: string): boolean;

  function DecimalString (theString: string): boolean;

  function StringToInteger (theString: string): longint;

  function StringToDecimal (theString: string): real;

  procedure ReadASmallInteger (var theInteger: integer);

  procedure GetBoundedInteger (lowerBound, upperBound: integer;
                  var theInteger: integer);

  procedure ReadANumber (var theNumber: real);

  procedure ReadBoundedNumber (lowerBound, upperBound: real;
                  var theNumber: real);

implementation

{_________________________________________________}

  procedure InitialiseGraphics (var SCREENBOX: IBOX;
                  var currentMode: integer);
      {This routine is specific to Turbo Pascal}

    var
      graphDriver, graphMode, CTX, CTY: integer;
      driverPath: string;

  begin
        {DetectGraph(graphDriver, graphMode);}
        {driverPath := 'C:\BP\BGI';}
        {driverPath := '..\PROGFILE\BGI';}
        {include BGI in PROGFILE: the program file}
        {graphDriver := detect;}
        {graphMode := 0;}

    driverPath := ' ';

       {graphDriver := D8bit; }
     {Alternative 1}
       {graphMode := m640x480;}
     {Alternative 1}

    graphDriver := detect;  {Alternative 2}
    DetectGraph(graphDriver, graphMode);   {Alternative 2}

    InitGraph(graphDriver, graphMode, driverPath);

    if GraphResult <> grOK then
      begin
        Writeln('    FATAL ERROR: graphics initialisation error');
        Halt(1);
      end;

    graphicsOn := true;
    SetBkColor(blue);
    SetColor(yellow);
    ClearDevice;
    currentMode := GetGraphMode;

    CTX := GetMaxX div 2;
    CTY := GetMaxY div 2;
    CTY := CTY - 20;

    if noSCREENBOX then
      begin
        SCREENBOX.X0 := SCREENBOX.X0 + CTX;
        SCREENBOX.Y0 := SCREENBOX.Y0 + CTY;
        SCREENBOX.X1 := SCREENBOX.X1 + CTX;
        SCREENBOX.Y1 := SCREENBOX.Y1 + CTY;
        SCREENBOX.X2 := SCREENBOX.X2 + CTX;
        SCREENBOX.Y2 := SCREENBOX.Y2 + CTY;
        SCREENBOX.XC := CTX;
        SCREENBOX.YC := CTY;

        noSCREENBOX := false;
      end;

  end; {InitialiseGraphics}

   {_________________________________________________}

  procedure GetGraphics (var graphicsOn: boolean;
                  var SCREENBOX: IBOX;
                  var currentMode: integer);
      {This routine is specific to Turbo Pascal}

  begin
    if graphicsOn then
      begin
        SetGraphMode(currentMode);
        SetBkColor(blue);
        SetColor(yellow);
        ClearDevice;
      end
    else
      InitialiseGraphics(SCREENBOX, currentMode);
        {graphicsOn := true;}
    graphicsOn := false;
  end; {GetGraphics}

   {_________________________________________________}

  procedure SetTheText;
     {This routine is specific to Turbo Pascal}

  begin
    TextBackGround(blue);
    ClrScr;
  end; {SetTheText}

   {__________________________________________________}

  procedure RevertText;
      {This routine is specific to Turbo Pascal}

  begin
    TextBackGround(black);
    ClrScr;
  end; {RevertText}

   {__________________________________________________}

  procedure CallForReturn;
      {This routine is specific to Turbo Pascal}

    var
      message: string;
      width, XS, YS: integer;
  begin
       {message := 'To Continue, Press <RETURN>';}
    message := 'To Continue, Close the Graph Window and Press <RETURN>';
    width := TextWidth(message);
    XS := (GetMaxX div 2) - (width div 2);
    YS := 10;
    MoveTo(XS, YS);
    OutText(message);
  end; {CallForReturn}

{________________________________________________}

  procedure SysBeep (pause: integer);

  begin
    Sound(220);
    Delay(pause * 30);
    NoSound;
  end; {SysBeep}

{________________________________________________}

  procedure ScreenCaption (legend: string);

    var
      i, q, width, distance, breakPoint, XS, XQ, YS: integer;
      firstString, secondString: string;

  begin
    XS := GetMaxX div 2;
    YS := (GetMaxY) - 30;

    width := TextWidth(legend);

    if width > 65 then
      begin {Breakit }
        distance := width;
        for i := 1 to width do
          begin {i}
            if legend[i] = ' ' then
              begin {if legend}
                q := Abs(width div 2 - i);
                if distance > q then
                  begin {if distance}
                    breakPoint := i;
                    distance := q;
                  end; {if distance}
              end; {if legend}
          end; {i}

        firstString := Copy(legend, 1, breakPoint);
        secondString := Copy(legend, breakPoint + 1, width);
        width := TextWidth(firstString);
        XQ := XS - (width div 2);
        YS := YS - 35;
        YS := SCREENBOX.Y2 + 60; {Alternative}
        MoveTo(XQ, YS);
        OutText(firstString);
        width := TextWidth(secondString);
        XQ := XS - (width div 2);
        YS := YS + 15;
        MoveTo(XQ, YS);
        OutText(secondString);
      end {Breakit}
    else
      begin
        width := TextWidth(legend);
        XS := XS - (width div 2);
        YS := YS - 30;
        YS := SCREENBOX.Y2 + 60; {Alternative}
        MoveTo(XS, YS);
        OutText(legend);
      end;

  end; {ScreenCaption}

{________________________________________________}

  procedure MultiCaption (var legend: string);

    var
      lineString: stringVector;
      width, breakPoint, nOfLines, desiredLength, nOfBreaks, j: integer;
      XS, YS, XQ: integer;
      breakFound: boolean;

  begin
    XS := GetMaxX div 2;
    YS := (GetMaxY) - 30;

    lineString[1] := legend;
    nOfLines := 1;
    width := TextWidth(legend);

    if width > 75 then
      begin {Breakit}
        nOfLines := (width div 70 + 1);
        desiredLength := width div nOfLines;
        nOfBreaks := 0;

        repeat {until nOfBreaks = nOfLines -1}
          j := -1;
          breakFound := false;
          repeat {until breakFound }
            j := j + 1;
            if legend[desiredLength - j] = ' ' then
              begin {if}
                breakFound := true;
                breakPoint := desiredLength - j;
              end; {if}
            if legend[desiredLength + j] = ' ' then
              begin {if}
                breakFound := true;
                breakPoint := desiredLength + j;
              end; {if}
          until breakFound;

          nOfBreaks := nOfBreaks + 1;
          lineString[nOfBreaks] := Copy(legend, 1, breakPoint);
          Delete(legend, 1, breakPoint - 1);
        until (nOfBreaks = nOfLines - 1);
        lineString[nOfLines] := legend;
      end; {Breakit }

    YS := YS - nOfLines * 15;
    YS := SCREENBOX.Y2 + 60; {Alternative}

    for j := 1 to nOfLines do
      begin
        width := TextWidth(lineString[j]);
        XQ := XS - (width div 2);
        MoveTo(XQ, YS);
        OutText(lineString[j]);
        YS := YS + 15;
      end

  end; {MultiCaption}

{_________________________________________________}

procedure PageParameters (var SCREENBOX, POSTBOX: IBOX);

  var
    CTX, CTY: integer; {these parameters repesent the centre of the screen}
    boxWidth, boxHeight: real;

begin {GraphParametersMod}
  Writeln('    Specify the frame surrounding the graph.');
  Writeln('    5cm x 3cm <= width x height <= 13.5cm x 9.5cm.');
  Writeln;
  Writeln('    For two diagrams per page use 9cm times 6cm');
  Writeln('    For three diagrams per page use 9cm times 3.75cm');
  Writeln;

  Writeln('              Specify the width ');
  ReadANumber(boxWidth);
  Writeln('              Specify the height ');
  ReadANumber(boxHeight);
  if boxHeight > 12.0 then
    boxHeight := 12.0;
  if boxHeight < 3 then
    boxHeight := 3;
  if boxWidth > 18.0 then
    boxWidth := 18.0;
  if boxWidth < 5 then
    boxWidth := 5;

  CTX := SCREENBOX.XC;{this is initally set to zero}
  CTY := SCREENBOX.YC; {this is initally set to zero}
  SCREENBOX.X0 := CTX;
  SCREENBOX.Y0 := CTY;
  SCREENBOX.X1 := CTX - Round((boxWidth * 36) / 2.54); {= HRS}
  SCREENBOX.Y1 := CTY - Round(boxHeight * 14.5); {= VTS}
  SCREENBOX.X2 := CTX + Round((boxWidth * 36) / 2.54); {= HRF}
  SCREENBOX.Y2 := CTY + Round(boxHeight * 14.5); {= VTF}
  SCREENBOX.BOXWDT :=  SCREENBOX.X2 - SCREENBOX.X1;
  SCREENBOX.BOXHGT :=  SCREENBOX.Y2 - SCREENBOX.Y1;

  POSTBOX.X1 := 0; {=XS}
  POSTBOX.Y1 := 0; {=YS}
  POSTBOX.X2 := 2 * Round((boxWidth * 36) / 2.54); {= XF}
  POSTBOX.Y2 := 2 * Round((boxHeight * 36) / 2.54); {= YF}
  POSTBOX.BOXWDT :=  POSTBOX.X2 - POSTBOX.X1;
  POSTBOX.BOXHGT :=  POSTBOX.Y2 - POSTBOX.Y1;


end; {Page Parameters}

{________________________________________________}

  procedure PageSetUp (var SCREENBOX, POSTBOX: IBOX;
                  var PSPict, TexFormat: Boolean);
  begin
    ClrScr;

    Writeln;
    Writeln('    SPECIFY THE PAGE PARAMETERS');
    Writeln;

    YesOrNo('    Do you want to output in PostScript', PSPict);
    if PSPict then
      YesOrNo('    Do you want a TeXtures Format', TeXFormat);

    PageParameters(SCREENBOX, POSTBOX);
    noPageParams := false;
  end; {PageSetUp}

{_________________________________________________}

  procedure TallyPoints (range: real;
                  var taly: real);

    var
      PWR, TRNC: integer;
      FACT, logBaseTen: real;

  begin {TallyPoints}
{TRNC contains the first two digits the range of Y}
{taly is the interval of the Tally marks}

    logBaseTen := Ln(range) / Ln(10);
    if logBaseTen > 0 then
      begin
        PWR := Trunc(logBaseTen) - 1;
        FACT := Exp(PWR * Ln(10));
        TRNC := Round(range / FACT)
      end;
    if logBaseTen = 0 then
      begin
        FACT := 0.1;
        TRNC := 10
      end;
    if logBaseTen < 0 then
      begin
        PWR := Trunc(Abs(logBaseTen)) + 2;
        FACT := Exp(PWR * Ln(10));
        TRNC := Round(range * FACT);
        FACT := 1 / FACT;
      end;
    if (TRNC >= 10) and (TRNC < 15) then
      taly := 2.5 * FACT;
    if (TRNC >= 15) and (TRNC < 30) then
      taly := 5 * FACT;
    if (TRNC >= 30) and (TRNC < 60) then
      taly := 10 * FACT;
    if (TRNC >= 60) and (TRNC <= 100) then
      taly := 20 * FACT;
  end; {TallyPoints}

{_________________________________________________}

  procedure YesOrNo (question: string;
                  var decision: boolean);
    var
      contrary: boolean;
      answer: char;
  begin
    repeat
      Writeln(question, ' Y/N?');
      Read(Answer);
      Readln;
      decision := (answer = 'Y') or (answer = 'y');
      contrary := (answer = 'N') or (answer = 'n');
    until (decision <> contrary);
  end; {YesOrNo}

{_________________________________________________}

  function UpCase (character: char): char;

  begin
    if character in ['a'..'z'] then
      UpCase := Chr(Ord(character) - Ord('a') + Ord('A'))
    else
      UpCase := character;
  end; {UpCase}

{_________________________________________________}

  function DownCase (character: char): char;

  begin
    if character in ['A'..'Z'] then
      DownCase := Chr(Ord(character) - Ord('A') + Ord('a'))
    else
      DownCase := character;
  end; {DownCase}

{_________________________________________________}

  procedure StripLeadingBlanks (var theString: string);
    var
      exit: boolean;
  begin
    exit := False;
    repeat
      if Length(theString) <> 0 then
        begin
          if (Ord(theString[1]) = SPACE) or (Ord(theString[1]) = TAB) then
            Delete(theString, 1, 1)
          else
            exit := True;
        end;
    until (Length(theString) = 0) or exit;
  end; {StripLeadingBlanks}

{_________________________________________________}

  procedure StripTrailingBlanks (var theString: string);
    var
      n, m: integer;
  begin {StripTrailingBlanks}
    n := Length(theString);
    if n <> 0 then
      begin {if n<>0}
        repeat
          m := n;
          if (Ord(theString[n]) = SPACE) or (Ord(theString[n]) = TAB) then
            begin
              Delete(theString, n, 1);
              m := n - 1;
            end;
          n := n - 1;
        until (m <> n);
      end; {if n<>0}
  end; {StripTrailingBlanks}

{_________________________________________________}

  procedure ReplaceWithin (var theString: string;
                  deletion, replacement: string);
    var
      a, n: integer;
  begin
    a := Pos(deletion, theString);
    n := Length(deletion);
    if a <> 0 then
      begin
        Delete(theString, a, n);
        Insert(replacement, theString, a);
      end;
  end; {ReplaceWithin}

{_________________________________________________}

  procedure ReplaceEveryCaseOf (deletion, replacement: string;
                  var theString: string);

    var
      a, n: integer;
      newString, tempString: string;

  begin
    newString := '';
    n := Length(deletion);
    repeat
      a := Pos(deletion, theString);
      if a <> 0 then
        begin
          tempString := Copy(theString, 1, a - 1);
          tempString := Concat(tempString, replacement);
          Delete(theString, 1, a + n - 1);
          newString := Concat(newString, tempString);
        end;
    until a = 0;
    theString := Concat(newString, theString);

  end; {ReplaceEveryCaseOf}

{_________________________________________________}

  procedure ReadAChar (var theCharacter: char);

    var
      charString: string;
  begin
    charString := '';
    repeat
      Readln(charString);
      StripLeadingBlanks(charString);
      StripTrailingBlanks(charString);
      if Length(CharString) <> 1 then
        begin
          Sysbeep(3);
          Writeln('    Type a single character!');
        end;
    until Length(CharString) = 1;
    theCharacter := charString[1];
  end; {ReadAChar}

{_________________________________________________}

  function IntegerToString (theInteger: integer): string;
    var
      i, n, digit: integer;
      theString, signChar: string;
  begin
    theString := '';
    signchar := '';
    if theInteger < 0 then
      begin
        signchar := '-';
        theInteger := -theInteger;
      end;

    if theInteger = 0 then
      theString := '0'
    else if (theInteger > 0) then
      begin {The Integer > 0}
        n := Trunc(Ln(theInteger) / Ln(10)) + 1;
        for i := 1 to n do
          begin {i}
            digit := theinteger mod 10;
            insert(Chr(digit + 48), thestring, 1);
            theinteger := theInteger div 10;
          end; {i}
      end; {The Integer > 0}
    theString := Concat(signChar, theString);
    IntegerToString := theString;
  end; {IntegerToString}

{_________________________________________________}

  function TalyToString (theNumber: real): string;
    var
      scientific, decimal: boolean;
      sign, leadingDigits, digits, divisor: integer;
      i, p, q, r: integer;
      realNumber, log10: real;
      headString, powerString, numbString: string;

  begin
  {0. Determine the sign of the number and take the modulus}
    p := 0;
    sign := 1;
    if theNumber < 0 then
      begin
        theNumber := -theNumber;
        sign := -1;
      end;

  {1. Extract leading digits and determine the value of the exponent p}
    if theNumber <> 0 then
      begin {theNumber <>0}
        digits := 4;
        log10 := Ln(theNumber) / Ln(10.0);
        p := Trunc(log10);
        if theNumber < 1 then
          p := p - 1;
        realNumber := theNumber / Exp((p - digits) * ln(10));
        leadingDigits := Round(realNumber / 10);
        if leadingDigits = 10000 then
          begin
            p := p + 1;
            leadingDigits := 1000;
          end;

    {2. Determine the number of nonzero digits}
        divisor := 10;
        q := 4;
        while leadingDigits mod divisor = 0 do
          begin
            divisor := divisor * 10;
            q := q - 1;
          end;
        r := q; {the original number of nonzero digits}
        if (p > 0) and (q < p + 1) then {not if p >4}
          q := Min(4, p + 1); {We need to count the integer zeros}

    {3. Determine whether or not to use Scientific notation}
        scientific := True;
        decimal := False;
        if (6 > (q - p)) and ((q - p) > 0) then
          begin
            scientific := False;
            decimal := True;
          end;
      end; {theNumber <>0}

  {4. Construct the Scientific  notation}
    if scientific and (theNumber <> 0) then
      begin {Scientific Notation}
        q := r;
        while ((leadingDigits mod 10) = 0) do
          leadingDigits := leadingDigits div 10;
        numbString := IntegerToString(leadingDigits);
        if q > 1 then
          Insert('.', numbString, 2)
        else if q = 1 then
          numbString := Concat(numbString, '.0');
        if p <> 0 then
          begin {Scientific Notation}
            powerString := IntegerToString(p);
            if p > 0 then
              powerString := Concat('+', powerString);
            numbString := Concat(numbString, 'e', powerString);
          end;{Scientific Notation}
      end; {Scientific Notation}

  {5. Construct the Decimal  notation}
    if Decimal and (theNumber <> 0) then
      begin
        for i := 1 to 4 - q do
          begin
            if (leadingDigits mod 10 = 0) then
              leadingDigits := leadingDigits div 10;
          end;
        numbString := IntegerToString(leadingDigits);
        if (p in [0, 1, 2]) and (q > p + 1) then
          Insert('.', numbString, p + 2);
        if p < 0 then
          begin {p <0}
            headString := '0.';
            for i := 2 to -p do
              headString := Concat(headString, '0');
            numbString := Concat(headString, numbString);
          end; {p <0}
      end; {Decimal}

    if sign = -1 then
      numbString := Concat('-', numbString);
    if theNumber = 0 then
      numbString := '0';

    TalyToString := numbString;

  end; {TalyToString}

{_________________________________________________}

  function IntegerString (theString: string): boolean;

    var
      numeric: boolean;
      i, n: integer;

  begin {IntegerString}
    numeric := True;
    n := Length(theString);
    if n > 5 then
      numeric := False;
    if (not (theString[1] in ['0'..'9'])) and (not (theString[1] = '-')) then
      numeric := False;
    for i := 2 to n do
      begin {i}
        if (not (theString[i] in ['0'..'9'])) then
          begin
            numeric := False;
          end;
      end; {i}
    IntegerString := numeric;
  end; {IntegerString}

{_________________________________________________}

  function DecimalString (theString: string): boolean;

    var
      numeric: boolean;
      i, n, points: integer;

  begin {DecimalString}
    points := 0;
    numeric := True;
    n := length(theString);
    if n <= 0 then
      numeric := False
    else
      begin {n>0}
        if not (theString[1] in ['0'..'9', '.', '-']) then
          numeric := False;
        if theString[1] = '.' then
          points := points + 1;
        for i := 2 to n do
          begin {i}
            if not (theString[i] in ['0'..'9', '.']) then
              numeric := False;
            if theString[i] = '.' then
              points := points + 1;
          end; {i}
      end; {n>0}
    if points > 1 then
      numeric := False;
    DecimalString := numeric;
  end; {DecimalString: Is it a Decimal String?}

{_________________________________________________}

  function StringToInteger (theString: string): longint;

    var
      i, n, start: integer;
      thenumber: longint;
      negative: boolean;

  begin
    start := 1;
    negative := False;
    if theString[1] = '-' then
      begin
        negative := True;
        start := 2;
      end;
    n := Length(theString);
    theNumber := 0;
    for i := start to n do
      theNumber := theNumber * 10 + Ord(theString[i]) - Ord('0');
  {Usually Ord('0') = 48}
    StringToInteger := theNumber;
    if negative then
      StringToInteger := -theNumber;
  end; {StringToInteger}

{_________________________________________________}

  function StringToDecimal (theString: string): real;

    var
      i, n, p, d: integer;
      theNumber, denominator: real;
      negative: boolean;

  begin{StringToDecimal}
    negative := False;
    if theString[1] = '-' then
      begin
        negative := True;
        Delete(theString, 1, 1);
      end;

    n := length(theString);
    p := pos('.', theString);
    if p = 0 then
      p := n;
    d := n - p;

    theNumber := 0;
    if p <> n then
      begin
        Delete(theString, p, 1);
        n := n - 1;
      end;

    for i := 1 to n do
      theNumber := theNumber * 10 + Ord(thestring[i]) - Ord('0');
  {Usually ord('0') = 48}
    denominator := 1;

    for i := 1 to d do
      denominator := denominator * 10;

    StringToDecimal := theNumber / denominator;
    if negative then
      StringToDecimal := -theNumber / denominator;
  end; {StringToDecimal}

{_________________________________________________}

  procedure ReadASmallInteger (var theInteger: integer);

    var
      theString: string;
      theLongint: longint;
      itsAnInteger: boolean;

  begin
    itsAnInteger := False;
    repeat {until itsAnInteger;}
      repeat
        Readln(theString);
      until theString <> '';
      itsAnInteger := IntegerString(theString);
      if itsAnInteger = False then
        begin {this is a rubbish string}
          Sysbeep(3);
          Writeln('Type a positive integer not exceeding 20! ');
        end {this is a rubbish string}

      else {it is an integer String}

        begin {Convert the IntegerString}
          theLongint := StringToInteger(theString);
          if theLongint > 32767 then
            begin
              Sysbeep(3);
              Writeln('This integer  exceeds 32767! (Try again).');
              itsAnInteger := False;
            end
          else
            begin
              theInteger := theLongint;
              itsAnInteger := True;
            end;
        end; {Convert the IntegerString}

    until itsAnInteger;
  end; {ReadASmallInteger}

{_________________________________________________}

  procedure GetBoundedInteger (lowerBound, upperBound: integer;
                  var theInteger: integer);

    var
      theString: string;
      theLongint: longint;
      itsAnInteger, withinBounds: boolean;

{USES: IntegerString, StringToInteger}

  begin
  {Writeln('    Type an integer in the interval [', lowerBound : 1, ', ', upperBound : 3, '] ');}
    itsAnInteger := False;
    repeat {until itsAnInteger;}
      repeat
        Readln(theString);
      until theString <> '';
      itsAnInteger := IntegerString(theString);
      if itsAnInteger = False then
        begin {this is a rubbish string}
          Sysbeep(3);
          Writeln('    Type an integer in the interval [', lowerBound : 1, ', ', upperBound : 1, '] ');
          Writeln;
        end {this is a rubbish string}

      else {it is an integer String}

        begin {Convert the IntegerString}
          withinBounds := False;
          theLongint := StringToInteger(theString);
          if theLongint > upperBound then
            begin
              Sysbeep(3);
              Writeln('    This integer exceeds ', upperBound : 1, '! (Try again).');
            end
          else if theLongint < lowerBound then
            begin
              Sysbeep(3);
              Writeln('    This integer is less than ', lowerBound : 1, '! (Try again).');
            end
          else
            begin
              theInteger := theLongint;
              withinBounds := True;
            end;
        end; {Convert the IntegerString}
    until withinBounds;
  end; {GetBoundedInt}

{_________________________________________________}

  procedure ReadANumber (var theNumber: real);

    var
      theString: string;
      itsANumber: boolean;

  begin
    itsANumber := False;
    repeat
      Readln(theString);
      StripLeadingBlanks(theString);
      StripTrailingBlanks(theString);
      itsANumber := DecimalString(theString);
      if itsANumber = False then
        begin
          Sysbeep(3);
          Write('Type a number ! ');
        end;
    until itsANumber;
    theNumber := StringToDecimal(theString);
  end; {ReadANumber}

{_________________________________________________}

  procedure ReadBoundedNumber (lowerBound, upperBound: real;
                  var theNumber: real);

    var
      accepted: boolean;

  begin

    repeat
      ReadANumber(theNumber);

      accepted := False;
      if (theNumber < lowerBound) then
        begin
          Sysbeep(3);
          Writeln('    This number is less than the minimum value ', lowerBound: 5:2);
          Writeln('    Input a number between ', lowerBound: 5:2,' and ', upperBound: 5:2);
        end
      else if (theNumber > upperBound) then
        begin
          Sysbeep(3);
          Writeln('    This number is greater that the maximum value ', upperBound : 5 : 2);
          Writeln('    Input a number between ', lowerBound: 5:2,' and ', upperBound: 5:2);
        end
      else
        accepted := True;
    until accepted;

  end;{ReadBoundedNumber}

{_________________________________________________}

end. {UTILS: Unit 3 of CONTEXT.PAS}

